;********************************************************************
;*																	*
;*   Мини ОС реального времени										*
;*   http://easyelectronics.ru										*
;*																	*
;*	Автоматика управления дворниками для ВАЗ21213					*
;*																	*
;*   Кристал: ATTiny13                                              *
;*                                                                  *
;*   main.asm Главный модуль										*
;*																	*
;*   Структура проекта:												*
;*		main.asm	- главный модуль (основной цикл)				*
;*		interupt.asm -Модуль прерываний								*
;*		tasks.asm 	- Модуль процессов								*
;* 		taskq.asm	- Модуль очереди процессов						*
;*		timers.asm	- Модуль очереди таймеров						*
;*		data.inc	- Данные в программной памяти					*
;*		define.inc	- Константы и дифайны							*
;*																	*
;********************************************************************

.nolist
.include "tn13def.inc"
.list

.include "define.inc"
.include "makros.asm"

;====== Организация ОЗУ ======================
.dseg
TaskQuere:		.byte TaskQuereSize			;очередь задач
TimersQuere:	.byte TimersQuereSize * 3	;очередь таймеров
BtnStatus:		.byte 1		;статус кнопки
SwitchStatus:	.byte 1		;статус рычага
BtnTimer:		.byte 2		;таймер кнопки
Pause:			.byte 2		;время паузы при прерывистом режиме

;====== Начало программы =====================
.cseg
.org 0
	rjmp reset	;RESET - переход на начало программы при перезагрузке или включении	
	reti		;INT0
	reti		;PCINT0
	reti		;TIM0_OVF
	reti		;EE_RDY
	reti		;ANA_COMP
	rjmp TC0COMA;TIM0_COMPA
	reti		;TIM0_COMPB
	reti		;WDT
	reti		;ADC
;====== Список задач =========================
.include "data.inc"
;====== Прерывания ===========================	
.include "interupts.asm"	
;====== Инициализация ========================
reset:
	ldi tmp1, RAMEND
	out SPL, tmp1	;Начало стека на конец ОЗУ
	;очистка регистров
	ldi ZL, 30
	clr ZH
	dec ZL
	st Z, ZH
	brne PC-2	
	;настройка порта
	ldi tmp1, ST_DDR
	out DDRB, tmp1
	ldi tmp1, ST_PORTB
	out PORTB, tmp1
	;настройка таймера
	clr	tmp1
	out	TCCR0A, tmp1	;Таймер 0
	ldi tmp1, 0x02	;предделитель на 8
	out TCCR0B, tmp1
	ldi tmp1, 0x04
	out TIMSK0, tmp1 ;Прерывание по сравнению
	ldi tmp1, 150	;OCR0A
	out OCR0A, tmp1	;Таймер срабатывает раз 1 мс
	;установка специальных регистров
	clr zero		;Регистр с нулевым значением
	ldi tmp1, 0x01
	mov one, tmp1	;Регистр с еденичным значением
	
	ldi tmp1, LUP		;начальное значение в BtnStatus
	sts BtnStatus, tmp1
	ldi tmp1, P_0		;начальное значеие в SwitshStatus
	sts SwitchStatus, tmp1 
	ldi tmp1, low(PauseMax)
	sts Pause, tmp1
	ldi tmp1, high(PauseMax)
	sts Pause + 1, tmp1
	
	
	rcall InitTaskQuere		;Инициализация очереди задач
	rcall InitTimersQuery	;Инициализация очереди таймеров
	
	SetTaskM TS_Proces		;установка в очередь задачу Proces
	SetTaskM TS_ScanBtn		;установка в очередь задачу сканирования кнопки
	SetTaskM TS_ScanSwitch  ;установка в очередь задачу сканирования рычага
	

main:
	sei						;Разрешаем глобальные прерывания
	wdr						;Сброс сторожевого таймера
	rcall ProcesTaskQuere	;Обработка очереди задач
	rcall idle				;Простой ядра
	rjmp main
;====== Подпрограммы =========================
.include "taskq.asm"
.include "timers.asm"
.include "tasks.asm"
.include "procedure.asm"
