;********************************************************************
;*																	*
;*   Мини ОС реального времени										*
;*   http://easyelectronics.ru										*
;*																	*
;*	Автоматика управления дворниками для ВАЗ21213					*
;*																	*
;*   Кристал: ATTiny13                                              *
;*                                                                  *
;*   main.asm Главный модуль										*
;*																	*
;*   Структура проекта:												*
;*		main.asm	- главный модуль (основной цикл)				*
;*		tasks.asm 	- Модуль процессов								*
;* 		taskq.asm	- Модуль очереди процессов						*
;*		timers.asm	- Модуль очереди таймеров						*
;*		data.inc	- Данные в программной памяти					*
;*		define.inc	- Константы и дифайны							*
;*																	*
;********************************************************************

;== Задача 0 - простой ядра
idle: 
	nop
	ret
	
;== Задача 1 ==============
;Обработка состояния кнопки и рычага
Proces:
	;Сдавим задачу в очередь таймеров на повторный запуск через
	SetTimerM TS_Proces, PS_Time	;PS_Time милисекунд
	lds tmp1, BtnStatus				;в tmp1 состояние кнопки
	lds tmp2, SwitchStatus			;в tmp2 положение рычага
	
	;==============состояние кнопки LDN=================================
	cpi tmp1, LDN			;если состояние кнопки не LDN
	brne PrcsNxt			;идем дальше
	cp BtnLDawn, zero		;проверяем флаг начала режима длиного нажатия
	breq PrcsLDNExit		;если нет выходим
	mov BtnLDawn, zero		;иначе сбрасываем флаг начала длиного нажатия 
	SetTaskM TS_WtM_On		;запускаем мотор омывайки
	cpi tmp2, P_2			;и если не режим P_2 
	breq PrcsLDNExit		;то запускаем дворники через ManualWiperOn
	SetTimerM TS_WpM_On, ManualWiperOn	;секунд
	mov CircleFlag, One		;выставляем флаг цикла работы дворников для 
							;устранения конфликта работы в режиме P_1
	mov P0WpMOn, one		;и устанавливаем флаг работы дворников в режиме P_0
PrcsLDNExit:				;выходим из процедуры
	ret
	;===================================================================
PrcsNxt:
	cpi tmp1, LUP			;режимы UP и DN временые и нас не интересуют
	breq PrcsLUP		
	ret						;поэтому просто уходим
PrcsLUP:
	;=============состояние кнопки LUP==================================
	;вначале обработаем состояние после долго нажатия
	cp BtnLDawn, zero		;проверяем флаг начала режима длиного нажатия
	breq PrcsLUP01		;если нет то идем обрабатывать короткие нажатия
	mov BtnLDawn, zero		;иначе сбрасываем флаг начала длиного нажатия 
	SetTaskM TS_WtM_Off		;отключаем мотор омывайки
	cpi tmp2, P_2			;и если не режим P_2 
	breq PrcsLUP00		;то отключаем дворники через ManualWiperOff
	SetTimerM TS_WpM_Off, ManualWiperOff	;секунд
	SetTimerM TS_ClrCF, ManualWiperOff  ;и сброс флага работы дворников
PrcsLUP00:
	ret
PrcsLUP01:
	cp BtnCnt, zero			;если не было нажатий то в по режиму переключателя
	breq PrcsLUPNoClick
	cp BtnCnt, one
	brne PrcsLUPdblclc
	;одно короткое нажатие
	clr btnCnt
	cpi tmp2, P_2			;нет действий в режиме P_2
	breq PrcsLUPdblclcExit
	cpi tmp2, P_1
	breq PrcsLUPP1			
	mov P0WpMOn, one		;работа в режиме P_0	
	SetTaskM TS_WpM_On
	SetTimerM TS_WpM_Off, OneWiperTime
	SetTimerM TS_ClrCF, OneWiperTime
	ret
PrcsLUPP1:					;работа в режиме P_1
	lds XL, Pause			;уменьшаем время паузы
	lds XH, Pause + 1
	subi XL, low(Delta)
	sbci XH, high(Delta)
	cpi XH, high(PauseMin)
	brsh PrcsLUPP1_L1
	ldi XL, low(PauseMax)
	ldi XH, high(PauseMax)
PrcsLUPP1_L1:	
	sts Pause, XL
	sts Pause+1, XH
	ret
PrcsLUPdblclc:
	;два и более коротких нажатий
	clr BtnCnt
	SetTaskM TS_WtM_On
	SetTimerM TS_WtM_Off, AutoWaterTime
	cpi tmp2, P_2
	breq PrcsLUPdblclcExit
	mov CircleFlag, one
	SetTimerM TS_WpM_On, AutoWiperTimeON
	SetTimerM TS_WpM_Off, AutoWiperTimeOff
	SetTimerM TS_ClrCF, AutoWiperTimeOff
PrcsLUPdblclcExit:
	ret
	;нет нажатий
PrcsLUPNoClick:
	cpi tmp2, P_2
	breq PrcsLUPExit
	cpi tmp2, P_1
	breq PrcsLUPNoClcP1
	
	ret
PrcsLUPNoClcP1:
	cp CircleFlag, zero
	brne PrcsLUPExit
	mov CircleFlag, one
	SetTaskM TS_WpM_On
	SetTimerM TS_WpM_Off, OneWiperTime
	ldi OSWR, TS_ClrCF
	lds XL, Pause
	lds XH, Pause+1
	rcall SetTimer	
PrcsLUPExit:				;выходим из процедуры
	ret

;== Задача 2 ==============
;Сканирование состояния кнопки
ScanBtn:
	;Сдавим задачу в очередь таймеров на повторный запуск через
	SetTimerM TS_ScanBtn, SB_Time	;SB_Time милисекунд
	sbis PINB, BtnBit      ;определяем нажата ли кнопка?
	rjmp SBtnL01
	mov BtnSt, zero			;нажата
	rjmp SBtnL02
SBtnL01:
	mov BtnSt, one			;отжата
SBtnL02:
	lds tmp1, BtnStatus		;загрузим предидущее состояние кнопки
	cpi tmp1, LUP			;CASE BtnState
	breq SBtnLUP
	cpi tmp1, DN
	breq SBtnDN
	cpi tmp1, LDN
	breq SBtnLDN
;***** Короткое отжатие *****
SBtnUP:
	cp BtnSt, zero			;кнопка не нажата?
	breq SBtnUPL02			;проверим как долго не нажата
	cp btnFlag, zero		;кнопка в предидущию итерацию не нажималась?
	breq SBtnUPL01			;установим флаг изменения состояния кнопки
	ldi tmp1, DN			;установим BtnState = DN
	sts BtnStatus, tmp1
	clr BtnFlag				;сбросим флаг измениния состояния кнопки
	rcall StartTimer		;запустим таймр кнопки
	rjmp SBtnExit
SBtnUPL01:
	mov btnFlag, one		
	rjmp SBtnExit
SBtnUPL02:
	ldi XL, low(ClickOver)	;проверим как долго отжата кнопка
	ldi XH, high(ClickOver)
	rcall TimerOver
	cp return, zero			;еще время не прошло?
	breq SBtnExit			;выходим
	ldi tmp1, LUP			;иначе устанавливаем длиное отжатие (LUP)
	sts BtnStatus, tmp1
	clr BtnTimerFlag		;остановим таймер кнопки
	rjmp SBtnExit
;***** Длиное отжатие *****	
SBtnLUP:
	cp BtnSt, zero			;кнопка не нажата?
	breq SBtnExit			;на выход
	cp btnFlag, zero		;кнопка в предидущию итерацию не нажималась?
	breq SBtnLUPL01			;установим флаг изменения состояния кнопки
	ldi tmp1, DN			;установим BtnState = DN
	sts BtnStatus, tmp1
	clr BtnFlag				;сбросим флаг измениния состояния кнопки
	rcall StartTimer		;запустим таймeр кнопки
	rjmp SBtnExit
SBtnLUPL01:
	mov btnFlag, one		
	rjmp SBtnExit
;***** Короткое нажатие *****
SBtnDN:
	cp BtnSt, one			;кнопка нажата?
	breq SBtnDNL02			;проверим как долго нажата
	cp BtnFlag, zero		;в предидущию итерацию не отжималась?
	breq SBtnDNL01			;установим флаг изменения состояния кнопки
	ldi tmp1, UP			;установим BtnState = UP
	sts BtnStatus, tmp1
	inc BtnCnt				;увеличим счетчик количества коротких нажатий
	clr BtnFlag				;сбросим флаг измениния состояния кнопки
	rcall StartTimer		;запустим таймeр кнопки
	rjmp SBtnExit
SBtnDNL01:
	mov BtnFlag, one
	rjmp SBtnExit
SBtnDNL02:
	ldi XL, low(ClickOver)
	ldi XH, high(ClickOver)
	rcall TimerOver
	cp return, zero			;еще время не прошло?
	breq SBtnExit			;на выход
	ldi tmp1, LDN			;иначе устанавливаем длиное нажатие (LDN)
	sts BtnStatus, tmp1		
	clr BtnTimerFlag		;остановим таймер кнопки
	clr BtnCnt				;сбросим количества коротких нажатий кнопки
	mov BtnLDawn, one		;установим флаг начала длиного нажатия
	rjmp SBtnExit
;***** Длиное отжатие *****
SBtnLDN:
	cp BtnSt, one			;кнопка нажата?
	breq SBtnExit			;на выход
	cp BtnFlag, zero		;в предидущию итерацию не отжималась?
	breq SBtnLDNL01			;установим флаг изменения состояния кнопки
	ldi tmp1, LUP			;установим BtnState = LUP
	sts BtnStatus, tmp1		;(после долгого нажатия сразу длиное нажатие)
	clr BtnFlag				;сбросим флаг измениния состояния кнопки
	mov BtnLDawn, one		;установим флаг окончания длиного нажатия
SBtnLDNL01:
	mov BtnFlag, one	
SBtnExit:
	ret

;== Задача 3 ==============
;Сканирование состояния рычага управления дворников
ScanSwitch:
	SetTimerM TS_ScanSwitch, SS_Time ;SS_Time милисекунд
	sbic PINB, SwitchBit	;если рычаг в положении P_1
	rjmp SSwitch_L01		;то SwitchBit = 0
	ldi tmp1, P_1			;установим режим P_1
	rjmp SSWitch_Exit		;на выход
SSWitch_L01:
	cp P0WpMOn, one			;иначе проверяем флаг работы
	breq SSWitch_L02		;дворников в режиме P_0
	sbis PINB, CntrlBit	;и если нет и на контрольном бите 1
	rjmp SSWitch_L02
	ldi tmp1, P_2			;то установим режим P_2
	rjmp SSWitch_Exit
SSWitch_L02:
	ldi tmp1, P_0			;иначе режим P_0
SSWitch_Exit:
	sts SwitchStatus, tmp1	;сохраним результаты сканирования
	ret
	
;== Задача 4 ==============
;Включение мотора дворников
WiperMotorOn:
	sbi PORTB, WpMBit
	ret
	
;== Задача 5 ==============
;Отключение моторов дворников
WiperMotorOff:
	cbi PORTB, WpMBit
	ret
	
;== Задача 6 ==============
;Включение мотора омывайки
WaterMotorOn:
	sbi PORTB, WtMBit
	ret
	
;== Задача 7 ==============
;Отключение мотора омывайки
WaterMotorOff:
	cbi PORTB, WtMBit
	ret
	
;== Задача 8 ==============
;Сброс флага цикла работы дворника
ClrCicleFlag:
	clr CircleFlag
	clr P0WpMOn
	ret
